module EWorld

    class RouteWriter < EWorld::BaseWriter

        # Responsible for writing the routes.js file.
        # @return void
        def self.write(project, files_to_write)

            raise RuntimeError, "Expected project type to be: #{Blufin::Projects::TYPE_QUASAR}, instead got: #{project[Blufin::Projects::TYPE]}" unless project[Blufin::Projects::TYPE] == Blufin::Projects::TYPE_QUASAR
            @generated   = []
            @overwritten = []

            begin

                files_to_write.each do |file|
                    Blufin::Files::write_file(file, Blufin::Arrays::convert_string_to_line_array(get_blank_file_content))
                    @generated << file
                end

            rescue => e

                Blufin::Terminal::print_exception(e)

            end

            return @generated, @overwritten

        end

        private

        # Returns the contents of a blank file.
        # @return string
        def self.get_blank_file_content
            <<TEMPLATE
<template>
    <div>
        <base-title h="5">{{ $route.meta.title }}</base-title>
    </div>
</template>

<script>
export default {
    data() {
        return {};
    }
};
</script>

<style scoped type="text/scss" lang="scss"></style>
TEMPLATE
        end

    end

end